package org.objectweb.fractal.bf.connectors.ws;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.HashMap;
import java.util.logging.Logger;

import org.apache.cxf.aegis.databinding.AegisDatabinding;
import org.apache.cxf.frontend.ClientProxyFactoryBean;

/**
 * Some doc about dynamic client factory:
 * http://cwiki.apache.org/CXF20DOC/dynamic-clients.html
 */
public class SoapCxfInvocationHandler implements InvocationHandler {

	Logger log = Logger.getLogger(SoapCxfInvocationHandler.class
			.getCanonicalName());

	private final ClientProxyFactoryBean clientProxyfactoryBean;

	public SoapCxfInvocationHandler(String address, Class<?> clazz) {

		clientProxyfactoryBean = new ClientProxyFactoryBean();
		clientProxyfactoryBean.setServiceClass(clazz);
		clientProxyfactoryBean.setAddress(address);
		final AegisDatabinding aegisDatabinding = new AegisDatabinding();

		// with cxf 2.0.5
		HashMap<String, Object> props = new HashMap<String, Object>();
		props.put(AegisDatabinding.WRITE_XSI_TYPE_KEY, true);
		clientProxyfactoryBean.setProperties(props);
		// with cxf 2.1
		// AegisContext aegisCtx = new AegisContext();
		// aegisCtx.setReadXsiTypes(true);
		// aegisDatabinding.setAegisContext(aegisCtx);
		clientProxyfactoryBean.setDataBinding(aegisDatabinding);

	}

	/**
	 * @see java.lang.reflect.InvocationHandler#invoke(java.lang.Object,
	 *      java.lang.reflect.Method, java.lang.Object[])
	 */
	public Object invoke(Object proxy, Method method, Object[] args)
			throws Throwable {

		log.finer("Method " + method.getName() + " with args: "
				+ Arrays.deepToString(args));

		Object client = clientProxyfactoryBean.create();

		String methodName = method.getName();

		Object result = null;

		final Method remoteMethod = client.getClass().getMethod(methodName,
				method.getParameterTypes());
		log.finer("Will invoke method: " + remoteMethod
				+ " on cxf client proxy: " + client);
		result = remoteMethod.invoke(client, args);

		log.finer("Result retured by proxy: " + result);

		return result;
	}
}
