package org.objectweb.fractal.bf.connectors.ws;

import java.util.HashMap;

import org.apache.cxf.aegis.databinding.AegisDatabinding;
import org.apache.cxf.frontend.ServerFactoryBean;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.bf.AbstractSkeleton;

/**
 * This class uses CXF SimpleFrontEnd (see
 * {@link here http://incubator.apache.org/cxf/javadoc/latest/org/apache/cxf/frontend/ServerFactoryBean.html } )
 * to create a web-service from a base java class.
 * 
 * @author Valerio Schiavoni <valerio.schiavoni@gmail.com>
 */

public class WsServerBridge extends AbstractSkeleton implements
		WsServiceAttributes {

	ServerFactoryBean wsServerFactoryBean;

	String baseAddress;

	private boolean isServiceConfigured = false;

	public WsServerBridge() {
		wsServerFactoryBean = new ServerFactoryBean();
		final AegisDatabinding aegisDatabinding = new AegisDatabinding();
		HashMap<String, Object> props = new HashMap<String, Object>();
		props.put(AegisDatabinding.WRITE_XSI_TYPE_KEY, true);
		wsServerFactoryBean.setProperties(props);

		wsServerFactoryBean.getServiceFactory()
				.setDataBinding(aegisDatabinding);

	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsServiceAttributes#configureWsWith(java.lang.String,
	 *      java.lang.Class, java.lang.Object, String)
	 */
	public void configureWsWith(String address, Class<?> serviceClass) {
		this.baseAddress = address;

		wsServerFactoryBean.setAddress(address);
		wsServerFactoryBean.setServiceClass(serviceClass);

	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsServiceAttributes#finalizeConfig(java.lang.String)
	 */
	public void finalizeConfig(String itfName) {
		try {
			wsServerFactoryBean
					.setServiceBean(delegate.getFcInterface(itfName));
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}
		isServiceConfigured = true;

	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsServiceAttributes#startWs()
	 */
	public void startWs() {
		if (!isServiceConfigured) {
			throw new RuntimeException(
					"You cannot invoke startWs before finalizeConfig");
		} else {
			wsServerFactoryBean.create();
		}

	}
}
